#include "Arduino.h"
#include "FanController.h"

FanController::FanController(byte pin, unsigned int threshold)
{
	_pin = pin;
	_interruptPin = digitalPinToInterrupt(pin);
	_threshold = threshold;
}

void FanController::begin()
{
	static byte instance;
	_instance = instance;
	_instances[instance] = this;
	digitalWrite(_pin, HIGH);
	_attachInterrupt();
	instance++;
}

unsigned int FanController::getSpeed() {
	unsigned int elapsed = millis() - _lastMillis;
	if (elapsed > _threshold)
	{
		detachInterrupt(_interruptPin);
		double correctionFactor = 1000.0 / elapsed;
		_lastReading = correctionFactor * _halfRevs / 2 * 60;
		_halfRevs = 0;
		_lastMillis = millis();
		_attachInterrupt();
		return _lastReading;
	}
}

void FanController::_attachInterrupt()
{
	switch (_instance)
	{
	case 0:
		attachInterrupt(_interruptPin, _triggerExt0, FALLING);
		break;
	case 1:
		attachInterrupt(_interruptPin, _triggerExt1, FALLING);
		break;
	case 2:
		attachInterrupt(_interruptPin, _triggerExt2, FALLING);
		break;
	case 3:
		attachInterrupt(_interruptPin, _triggerExt3, FALLING);
		break;
	case 4:
		attachInterrupt(_interruptPin, _triggerExt4, FALLING);
		break;
	case 5:
		attachInterrupt(_interruptPin, _triggerExt5, FALLING);
		break;
	}
}

FanController * FanController::_instances[6];

void FanController::_trigger()
{
	_halfRevs++;
}

void FanController::_triggerCaller(byte instance)
{
	if (FanController::_instances[instance] != nullptr)
	{
		FanController::_instances[instance]->_trigger();
	}
}

void FanController::_triggerExt0() { FanController::_triggerCaller(0); }
void FanController::_triggerExt1() { FanController::_triggerCaller(1); }
void FanController::_triggerExt2() { FanController::_triggerCaller(2); }
void FanController::_triggerExt3() { FanController::_triggerCaller(3); }
void FanController::_triggerExt4() { FanController::_triggerCaller(4); }
void FanController::_triggerExt5() { FanController::_triggerCaller(5); }